/*
	* Copyright (c) 2021-11 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
	* 
	* App to implement communication ESP32 <-> uniTFT
	* Support uniTFT example project weather, keyboard and direct connection WiFi and Bluetooth
	*
	* This file is free software; you can redistribute it and/or modify
	* it under the terms of either the GNU General Public License version 2
	* or the GNU Lesser General Public License version 2.1, both as
	* published by the Free Software Foundation.
*/

#include "debug.h" //Debug output to serial port ESP32
#include "src/SmallProtocol/uniTFT.h"
#include "src/WiFi/wifi_ea.h"
#include "bt_keyboard.h"
#include "weather.h"

Weather weather; //Class weather to support uniTFTDesigner example project
BT_Keyboard bt_keyboard; //Class bluetooth to support uniTFTDesigner example project


void decodeCommands(); //called to decode data, sent from uniTFT

char strBuf[64]; //local StringBuffer


/*----------------------------
	Func: setup
	Desc: Initializes project: Debug, uniTFT connection, Weather, keyboard
	Vars: ---
	Ret : ---
------------------------------*/
void setup() 
{
	#if DBG_ON
		Serial.begin(115200);
		Serial.printf("Starting EA uniTFTs ESP32 Demo Version %0.1f\r\n",VERSION);
		Serial.flush();
	#endif
	uniTFT.initialize_rs232(BAUDUNITFT,32); //connection uniTFT, use RS232, used in EA 94998-E (Demopack WiFi)
	//uniTFT.initialize_spi(SPISettings(400000, MSBFIRST, SPI_MODE3),32); //connection uniTFT, use SPI; untested, customer specific hardware needed.
	esp_reset_reason_t reason = esp_reset_reason(); //get Reset reason. Used to change bluetooth mode (Serial <-> Keyboard)
	bt_keyboard.setup(reason);  //initialize bluetooth class
	WiFi_EA.setup();
}

unsigned long myTime = 0;
/*----------------------------
	Func: loop
	Desc: Main loop
	Vars: ---
	Ret : ---
------------------------------*/
void loop() 
{

	if(!bt_keyboard.isConnectedAsSerial()) //if bluetooth connection is not serial (direct connection, data transfer uniTFT to bluetooth serial port)
	{
		uniTFT.check_sendBuffer(); //check if uniTFT has data present
		if(uniTFT.dataAvailable() > 3) //command completly received (data present)
			decodeCommands(); //decode data
	}
	bt_keyboard.checkTask(); //always check if something is to do with bluetooth
	WiFi_EA.WiFiServerReceivedData(); //alwais check if something is to do with WiFi connection (e.g. TCP packet).
	if(WiFi_EA.wifi_reconnect_needed())
		WiFi_EA.connectWiFi(WiFi_EA.userSSID(), WiFi_EA.userPassword());
	
}

/*----------------------------
	Func: decodeCommands
	Desc: Decoding incoming data from uniTFT
	Vars: ---
	Ret : ---
------------------------------*/
void decodeCommands()
{
	int len = uniTFT.dataAvailable();
	byte command[4];
	#if (DBG_ON && DBG_MAIN)
		Serial.println("Decoding commands");
	#endif
	if(len>3) //Answer at least 4 bytes: ESC + 3 chars command
	{
		command[0] = uniTFT.getByte(); command[1] = uniTFT.getByte(); command[2] = uniTFT.getByte(); command[3] = uniTFT.getByte(); //get the first 4 bytes to decode
		if(command[0] != ESC)
		{
			uniTFT.resetReceiveBuf();
		#if (DBG_ON && DBG_MAIN)
			Serial.printf("Command: 0x%02X 0x%02X 0x%02X 0x%02X\r\n", command[0],command[1],command[2],command[3]);
		#endif
			return;
		}
		#if (DBG_ON && DBG_MAIN)
			Serial.printf("Command: 0x%02X%c%c%c\r\n", command[0],command[1],command[2],command[3]);
		#endif
        if(command[1] == '?') //Individual command by project. Standard responses using A-Z, we use ? for user defined interaction
        {
			if(command[2] == 'B') //Bluetooth command
				bt_keyboard.decodeCommands(command[3]); //decode in class 
			else if(command[2] == 'W') //WiFi or weather command
			{
				switch(command[3])
				{
				case 'C': //Connect WiFi
				{
					char ssid[64];
					char password[64];
					uniTFT.getString(ssid);
					uniTFT.getString(password);
					
					sprintf(strBuf, "#MPT1,0\x0a"); //Stop macro process 1, hour glass for connection to 
					uniTFT.sendDataPacket(strBuf,strlen(strBuf));
					
					#if (DBG_ON && DBG_MAIN)
						Serial.println("connect to WiFi"); 
						Serial.print(ssid); Serial.print("  "); Serial.println(password); 
					#endif	
					bt_keyboard.BT_off(); //Do not use WiFi and bluetooth in parallel, so switch off bluetooth.
					WiFi_EA.connectWiFi(ssid, password); //connect to WiFi
					#if (DBG_ON && DBG_MAIN)
						Serial.printf("Connection to %s connected: %d\r\n", ssid, WiFi_EA.isConnected());
						Serial.print("IP: "); Serial.println(WiFi_EA.localIP());
					#endif
				}break;
				case 'R': //Reset ESP32
					ESP.restart();
				break;
				default: //otherwise, decode in class weather
					weather.decodeCommands(command[3]);
				break;
				}
			}
		}
	}
}
