/*
	* Copyright (c) 2021-11 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
	*
	* Weather implementation ESP32 and demo project EA uniTFTs
	* Weatherdata from https://openweathermap.org/
	*
	* This file is free software; you can redistribute it and/or modify
	* it under the terms of either the GNU General Public License version 2
	* or the GNU Lesser General Public License version 2.1, both as
	* published by the Free Software Foundation.
*/
#ifndef WEATHER_H
#define WEATHER_H

#include <Arduino.h>

#define WEATHERELEMENTS 31 //Amount of weather icons, like rainy, sunny, cloudy and so on.


struct weathericon //structure for weather info images
{
  int id_start;
  int id_end;
  char picture[20] ;
};

class Weather
{
public:
	void decodeCommands(byte command); //decode commands concerning weather data
  
private:  
	static const weathericon lookup[WEATHERELEMENTS];  //look-up table for icons depending on weather situation
	char szStr[300]; //Stringbuffer ascii
	char16_t unicodeStr[600]; //Stringbuffer unicode
	
	void getNetworkList(); //Find WiFi nearby
	
	//Commands for uniTFTs
	void changeRegister(int id, int value);
	void changeRegister(int id, float value);
	void changeStringRegister(int id, const char *text);
	void changeStringRegisterUnicode(int id, const char *text);
	void changeArrayValue(int id, int index, int value);
	void changeArrayValue(int id, int index, float value);
	void changeArrayValue(int id, int index, const char *text);

	void changeObjectVisibility(int id, int visibility);	
	void runMacro(const char *macroName);
	void updateWeather();
	void updateWeatherForecast();

	//Functions to work with openweathermap.org
	bool checkWeather();
	bool checkForecast();
	void findWeatherPic(char* weatherPic, int weatherId);
	bool getStringForGPSLocation(char *latitude, char *longitude, char *language, char unit, char *appid);
	bool getStringForLocation(char16_t *location, char *language, char unit, char *appid);
	bool getStringForGPSLocationForecast(char *latitude, char *longitude, char *language, char unit, char *appid);
	bool getStringForLocationForecast(char16_t *location, char *language, char unit, char *appid);
};

#endif //#define WEATHER_H
